# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'easy_type/flexible_fact'

EasyType::FlexibleFact.define(:db2_install_locations) do
  def db2ls
    @db2ls ||= Facter::Util::Resolution.exec('/usr/local/bin/db2ls -c') || ''
  end

  def db2ls_features(location)
    @db2ls_features ||= Facter::Util::Resolution.exec("/usr/local/bin/db2ls -c -q -b #{location}") || ''
  end

  def locations
    data_lines = db2ls.split("\n").drop(1)
    data_lines.collect do |line|
      line.split(':')[0]
    end
  end

  def version_of(location)
    data_lines = db2ls.split("\n").drop(1)
    data_lines.filter_map do |line|
      entries = line.split(':')
      next unless entries[0] == location

      entries[1]
    end.first
  end

  def product_version(location)
    full_version = version_of(location)
    version_components = full_version.split('.')
    {
      :full_version => full_version,
      :short_version => "#{version_components[0]}.#{version_components[1]}",
      :major_version => version_components[0],
      :minor_version => version_components[1],
      :fixpack => version_components[2]
    }
  end

  def installed_components(location)
    data_lines = db2ls_features(location).split("\n").drop(4)
    data_lines.map do |line|
      entries = line.split(':')
      {
        entries[0].downcase.to_sym => {
          :version => entries[1],
          :fixpack => entries[2].to_i,
          :description => entries[3].strip
        }
      }
    end
  end

  components locations

  part(:product_version) do |location|
    product_version(location)
  end

  part(:installed_components) do |location|
    installed_components(location)
  end
end
