define db2_install::private::install_fixpack (
  String[1]                            $version,
  Stdlib::Absolutepath                 $temp_dir,
  String[1]                            $source,
  String[1]                            $file_name,
  Stdlib::Absolutepath                 $location,
  Variant[Boolean, Enum['on_failure']] $logoutput      = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                              $allow_insecure = lookup({ name => 'allow_insecure', default_value => false }),
) {
  db2_install { "private::install_fixpack::${title}": ensure => 'present' }

  $sanitized_name = regsubst($title, / |\/|-|\./, '_', 'G')
  $requested_fixpack = $version.split('\.')[2]
  $installed_fixpack = $facts.dig('db2_install_locations','product_version', $location, 'fixpack')

  easy_type::debug_evaluation() # Show local variable on extended debug

  file { "${temp_dir}/${sanitized_name}":
    ensure => 'directory',
  }

  -> archive { "${temp_dir}/${file_name}":
    ensure         => present,
    allow_insecure => $allow_insecure,
    cleanup        => true,
    extract        => true,
    extract_path   => "${temp_dir}/${sanitized_name}",
    source         => "${source}/${file_name}",
    temp_dir       => $temp_dir,
  }

  if $requested_fixpack > String($installed_fixpack) {
    exec { "Apply ${title}":
      command   => "${temp_dir}/${sanitized_name}/universal/installFixPack -n -y -b ${location}",
      logoutput => $logoutput,
      require   => Archive["${temp_dir}/${file_name}"],
    }
  } else {
    exec { "Apply ${title} with downgrade":
      command   => "${temp_dir}/${sanitized_name}/universal/installFixPack -n -y -b ${location} -f level",
      logoutput => $logoutput,
      require   => Archive["${temp_dir}/${file_name}"],
    }
  }

  cleanup { "Temp files for ${title}":
    file_name => [
      "${temp_dir}/${sanitized_name}",
      "${temp_dir}/${file_name}",
    ],
  }
}
